/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDAlbum.h"
#import "MPDUtils.h"
#import "../MPDSocket.h"
#import "../Error/MPDServerError.h"
#import "../../Model/Song.h"


@implementation MPDAlbum

static MPDAlbum *sMPDAlbumManager = nil;

// Singleton
+ (MPDAlbum *)sMPDAlbum {
    if (sMPDAlbumManager == nil)
        sMPDAlbumManager = [[super allocWithZone:NULL] init];
    
    return sMPDAlbumManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDAlbum];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id)init {
    self = [super init];
    return self;
}

- (NSArray *)getAlbumsList {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        
        NSMutableArray *pathList = [NSMutableArray array];
        
        mpd_send_list_all([[MPDSocket sMPDSocket] getConnection], "/");
        
        struct mpd_entity * entity;
        
        // get all direcrectory contain song
        [self getAllDirectory:&entity pathList:pathList];
        
        return [pathList deleteDuplicateAlbum:[[MPDSocket sMPDSocket] getConnection]].sortAlbum;
    }
    
    return nil;
}

- (void)getAllDirectory:(struct mpd_entity **)entity pathList:(NSMutableArray *)pathList {
    while ((*entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
        const struct mpd_song *song;
        
        if (mpd_entity_get_type(*entity) == MPD_ENTITY_TYPE_SONG) {
            song = mpd_entity_get_song(*entity);
            
            NSString *list = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
            
            // Get path only
            NSString * plop = [list stringByDeletingLastPathComponent];
            
            [pathList addObject:plop];
        }
        
        mpd_entity_free(*entity);
    }
}


- (NSArray *)getAlbumContent:(Album *)aAlbum {
    
    if ([[MPDSocket sMPDSocket] isConnected]) {
        
        NSMutableArray *albumContent = [NSMutableArray array];
        NSString *p = aAlbum.path;
        const char *path = [p UTF8String];
        struct mpd_entity * entity;
        
        if (!mpd_send_list_meta([[MPDSocket sMPDSocket] getConnection], path)) {
            [[MPDServerError sMPDServerError] send:[[MPDSocket sMPDSocket] getConnection] ];
            return nil;
        }
        
        [self getAllSongs:albumContent entity:&entity];
        return albumContent;
    }
    return nil;
}

- (void)getAllSongs:(NSMutableArray *)albumContent entity:(struct mpd_entity **)entity {
    while ((*entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
        const struct mpd_song *song;
        
        if (mpd_entity_get_type(*entity) == MPD_ENTITY_TYPE_SONG) {
            song = mpd_entity_get_song(*entity);
            [albumContent addObject:[self getSongValues:song]];
        }
        mpd_entity_free(*entity);
    }
}

- (Song *)getSongValues:(const struct mpd_song *)song {
    
    NSString *songPath = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
    
    const struct mpd_audio_format *format = mpd_song_get_audio_format(song);
    NSString *extension = [songPath pathExtension];
    
    Song *s = [[Song alloc]init];
    [s setComposer:[GetTag song:song tag:MPD_TAG_COMPOSER]];
    [s setAlbumArtist:[GetTag song:song tag:MPD_TAG_ALBUM_ARTIST]];
    [s setArtist:[GetTag song:song tag:MPD_TAG_ARTIST]];
    [s setAlbum:[GetTag song:song tag:MPD_TAG_ALBUM]];
    [s setTitle:[GetTag song:song tag:MPD_TAG_TITLE]];
    [s setGenre:[GetTag song:song tag:MPD_TAG_GENRE]];
    [s setComment:[GetTag song:song tag:MPD_TAG_COMMENT]];
    [s setDuration:mpd_song_get_duration(song)];
    [s setTrack:[[GetTag song:song tag:MPD_TAG_TRACK] intValue]];
    [s setDate:[[GetTag song:song tag:MPD_TAG_DATE] intValue]];
    [s setDiskNb:[[GetTag song:song tag:MPD_TAG_DISC] intValue]];
    [s setFullPath:songPath];
    [s setExtension:extension];
    
    if ([extension isEqualToString:@"dsf"] || [extension isEqualToString:@"dff"])
        [s setBits:1];
    else
        [s setBits:format->bits];
    
    return s;
}

@end
