/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDPlayLists.h"
#import "MPDSocket.h"
#import "MPDUtils.h"
#import "../Error/MPDServerError.h"
#import "../../Model/Song.h"


@implementation MPDPlayLists

static MPDPlayLists *sMPDPlayListsManager = nil;

// Singleton
+ (MPDPlayLists *)sMPDPlayLists {
    if (sMPDPlayListsManager == nil)
        sMPDPlayListsManager = [[super allocWithZone:NULL] init];
    
    return sMPDPlayListsManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDPlayLists];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id)init {
    self = [super init];
    return self;
}


- (NSArray *)getPlaylists {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        
        NSMutableArray *playlists = [NSMutableArray array];
        struct mpd_entity * entity;
        const struct mpd_playlist *pl;
        
        if (!mpd_send_list_meta([[MPDSocket sMPDSocket] getConnection], "/")) {
            [[MPDServerError sMPDServerError] send:[[MPDSocket sMPDSocket] getConnection] ];
            return nil;
        }
        
        while ((entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
            
            if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_PLAYLIST) {
                pl = mpd_entity_get_playlist(entity);
                
                if (strncmp(mpd_playlist_get_path(pl),"[Radio Streams]", 15) != 0) {
                    
                    NSString *list = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_playlist_get_path(pl) encoding:NSUTF8StringEncoding]];
                    
                    [playlists addObject:list];
                }
            }
            
            mpd_entity_free(entity);
        }
        
        return playlists.sortList;
    }
    
    return nil;
}


- (NSArray *)getPlaylist:(NSString *)aPlist {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        
        NSMutableArray *playListContentPath = [NSMutableArray array];
        const char *pth = [aPlist UTF8String];
        const char *rara = nil;
        const struct mpd_playlist *pl;
        struct mpd_entity * entity;
        
        mpd_send_list_meta([[MPDSocket sMPDSocket] getConnection], "/");
        
        // get playlist format
        while ((entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
            
            if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_PLAYLIST) {
                pl = mpd_entity_get_playlist(entity);
                
                if (strncmp(mpd_playlist_get_path(pl),pth, strlen(pth)) == 0)
                    rara = mpd_playlist_get_path(pl);
            }
        }
        
        // list playlist content
        mpd_send_list_playlist([[MPDSocket sMPDSocket] getConnection], rara);
        
        while ((entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
            const struct mpd_song *song;
            
            if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_SONG) {
                song = mpd_entity_get_song(entity);
                
                NSString *plop = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
                [playListContentPath addObject:plop];
            }
        }
        
        return [self finalList:&entity playListContentPath:playListContentPath];
    }
    
    return nil;
}

- (NSArray *)finalList:(struct mpd_entity **)entity playListContentPath:(NSArray *)playListContentPath {
    
    NSMutableArray *playList = [NSMutableArray array];
    
    for (NSString *obj in playListContentPath) {
        
        mpd_send_list_meta([[MPDSocket sMPDSocket] getConnection], [obj UTF8String]);
        
        while ((*entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
            const struct mpd_song *song;
            
            if (mpd_entity_get_type(*entity) == MPD_ENTITY_TYPE_SONG) {
                song = mpd_entity_get_song(*entity);
                
                NSString *art = [GetTag song:song tag:MPD_TAG_ALBUM_ARTIST];
                NSString *path = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
                
                if (art == nil)
                    art = [GetTag song:song tag:MPD_TAG_ARTIST];
                
                Song *s = [[Song alloc]init];
                [s setComposer:[GetTag song:song tag:MPD_TAG_COMPOSER]];
                [s setArtist:art];
                [s setAlbum:[GetTag song:song tag:MPD_TAG_ALBUM]];
                [s setTitle:[GetTag song:song tag:MPD_TAG_TITLE]];
                [s setTrack:[[GetTag song:song tag:MPD_TAG_TRACK] intValue]];
                [s setDate:[[GetTag song:song tag:MPD_TAG_DATE] intValue]];
                [s setGenre:[GetTag song:song tag:MPD_TAG_GENRE]];
                [s setDuration:mpd_song_get_duration(song)];
                [s setFullPath:path];
                
                [playList addObject:s];
            }
            mpd_entity_free(*entity);
        }
    }
    
    return playList;
}


- (void)renamePlaylist:(NSString *)oldName newName:(NSString *)newName {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        const char *old = [oldName UTF8String];
        const char *new = [newName UTF8String];
        mpd_run_rename([[MPDSocket sMPDSocket] getConnection], old, new);
    }
}

- (void)newPlaylist:(NSString *)playlist {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        const char *list = [playlist UTF8String];
        mpd_run_playlist_add([[MPDSocket sMPDSocket] getConnection], list, "");
        mpd_run_playlist_clear([[MPDSocket sMPDSocket] getConnection], list);
    }
}

- (void)deletePlaylist:(NSString *)playlist {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        const char *list = [playlist UTF8String];
        mpd_run_rm([[MPDSocket sMPDSocket] getConnection], list);
    }
}

@end
