/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDQueue.h"
#import "MPDSocket.h"
#import "MPDUtils.h"
#import "../Error/MPDServerError.h"
#import "../../Model/Song.h"


@implementation MPDQueue

static MPDQueue *sMPDQueueManager = nil;

// Singleton
+ (MPDQueue *)sMPDQueue {
    if (sMPDQueueManager == nil)
        sMPDQueueManager = [[super allocWithZone:NULL] init];
    
    return sMPDQueueManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDQueue];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id)init {
    self = [super init];
    return self;
}


- (NSArray *)getQueue {
    
    if ([[MPDSocket sMPDSocket] isConnected]) {
        
        struct mpd_status * status;
        mpd_send_status([[MPDSocket sMPDSocket] getConnection]);
        status = mpd_recv_status([[MPDSocket sMPDSocket] getConnection]);
        
        if (status == NULL) {
            [[MPDServerError sMPDServerError] send:[[MPDSocket sMPDSocket] getConnection] ];
            return nil;
        } else {
            int queueLenght = mpd_status_get_queue_length(status);
            return [self getSongs:queueLenght];
        }
    }
    return nil;
}


- (NSArray *)getSongs:(int)queueLenght {
    NSMutableArray *queue = [NSMutableArray array];
    struct mpd_song *song;
    unsigned int i = 0;
    
    while (i < queueLenght) {
        
        song = mpd_run_get_queue_song_pos([[MPDSocket sMPDSocket] getConnection], i);
        i++;
        
        NSString *path = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
        
        Song *s = [[Song alloc]init];
        [s setComposer:[GetTag song:song tag:MPD_TAG_COMPOSER]];
        [s setArtist:[GetTag song:song tag:MPD_TAG_ARTIST]];
        [s setAlbum:[GetTag song:song tag:MPD_TAG_ALBUM]];
        [s setAlbumArtist:[GetTag song:song tag:MPD_TAG_ALBUM_ARTIST]];
        [s setTitle:[GetTag song:song tag:MPD_TAG_TITLE]];
        [s setTrack:[[GetTag song:song tag:MPD_TAG_TRACK] intValue]];
        [s setDate:[[GetTag song:song tag:MPD_TAG_DATE] intValue]];
        [s setGenre:[GetTag song:song tag:MPD_TAG_GENRE]];
        [s setDuration:mpd_song_get_duration(song)];
        [s setFullPath:path];
        
        [queue addObject:s];
        mpd_song_free(song);
    }
    
    return queue;
}


- (void)addToQueue:(NSArray *)array {
    
    if ([[MPDSocket sMPDSocket] isConnected]) {
        
        for (Song *song in array) {
            const char *file = [song.fullPath UTF8String];
            mpd_run_add([[MPDSocket sMPDSocket] getConnection], file);
        }
    }
}


- (void)removeInQueue:(NSArray *)array {
    
    if ([[MPDSocket sMPDSocket] isConnected]) {
        unsigned int i = 0;
        for (NSString *position in array) {
            mpd_run_delete([[MPDSocket sMPDSocket] getConnection], [position intValue] - i);
            i++;
        }
    }
}


- (void)saveQueueAtPlaylist:(NSString *)playlist {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        const char *list = [playlist UTF8String];
        mpd_run_save([[MPDSocket sMPDSocket] getConnection], list);
    }
}


@end
