/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "MPDStream.h"
#import "MPDSocket.h"
#import "../Error/MPDServerError.h"
#import "../../Model/Stream.h"


@implementation MPDStream

static MPDStream *sMPDStreamManager = nil;

// Singleton
+ (MPDStream *)sMPDStream {
    if (sMPDStreamManager == nil)
        sMPDStreamManager = [[super allocWithZone:NULL] init];
    
    return sMPDStreamManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sMPDStream];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id)init {
    self = [super init];
    return self;
}


- (NSArray *)getStreamList {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        NSMutableArray *radio = [NSMutableArray array];
        
        struct mpd_entity * entity;
        const struct mpd_playlist *pl;
        const char *rara = nil;
        
        if (!mpd_send_list_meta([[MPDSocket sMPDSocket] getConnection], "/")) {
            [[MPDServerError sMPDServerError] send:[[MPDSocket sMPDSocket] getConnection] ];
            return nil;
        }
        
        while ((entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
            
            if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_PLAYLIST) {
                pl = mpd_entity_get_playlist(entity);
                
                if (strncmp(mpd_playlist_get_path(pl),"[Radio Streams]", 15) == 0)
                    rara = mpd_playlist_get_path(pl);
            }
        }
        
        
        mpd_send_list_playlist([[MPDSocket sMPDSocket] getConnection], rara);
        
        int i = 0;
        while ((entity = mpd_recv_entity([[MPDSocket sMPDSocket] getConnection])) != NULL) {
            const struct mpd_song *song;
            
            if (mpd_entity_get_type(entity) == MPD_ENTITY_TYPE_SONG) {
                song = mpd_entity_get_song(entity);
                
                NSString *list = [NSString stringWithFormat:@"%@", [NSString stringWithCString:mpd_song_get_uri(song) encoding:NSUTF8StringEncoding]];
                
                NSRange rangeOfDiese = [list rangeOfString:@"#"];
                NSString *url = [list substringToIndex:rangeOfDiese.location];
                NSString *name = [list substringFromIndex:rangeOfDiese.location + 1];
                
                Stream *s = [[Stream alloc]init];
                [s setUrl:url];
                [s setName:name];
                [s setPosition:i];
                
                [radio addObject:s];
                
                i++;
            }
            mpd_entity_free(entity);
        }
        return [radio copy];
    }
    return nil;
}

- (void)addStream:(NSString *)aStream {
    if ([[MPDSocket sMPDSocket] isConnected]) {
        const char *url = [aStream UTF8String];
        mpd_run_playlist_add([[MPDSocket sMPDSocket] getConnection], "[Radio Streams]", url);
    }
}

- (void)deleteStream:(unsigned int)position {
    if ([[MPDSocket sMPDSocket] isConnected])
        mpd_run_playlist_delete([[MPDSocket sMPDSocket] getConnection], "[Radio Streams]", position);
}

@end
