/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <include/mpd/status.h>
#import "Status/MPDStatus.h"
#import "../Model/Radio.h"


// Main activity update
@protocol MPDAppDelegate <NSObject>
- (void)onProgressChange:(MPDStatus *)status;
- (void)onAlbumUpdateTrigger:(NSString *)msg;
- (void)onStateChange:(int)state;
- (void)onTrackChange:(MPDStatus *)radio;
- (void)onVolumeChange:(unsigned)volume;

@end

@protocol MPDRadioDelegate <NSObject>
- (void)onRadioLinkClick:(Radio *)radio;
@end

@protocol MPDQueueDelegate <NSObject>
- (void)onStateChange:(int)state position:(int)position;
- (void)onTrackChange:(int)position;
@end

@protocol MPDStreamDelegate <NSObject>
- (void)onStreamChange;
@end

// update Library view
@protocol MPDAlbumDelegate <NSObject>
- (void)onDbUpdate;
@end

@interface MPDCache : NSObject

@property (nonatomic, strong) id<MPDAppDelegate> delegate;
@property (nonatomic, strong) id<MPDAlbumDelegate> delegateAlbum;
@property (nonatomic, strong) id<MPDQueueDelegate> delegateQueue;
@property (nonatomic, strong) id<MPDStreamDelegate> delegateStream;
@property (nonatomic, strong) id<MPDRadioDelegate> delegateRadio;

+ (MPDCache *)sMPDCache;

- (void)setStatus:(MPDStatus *)mpdStatus;
- (void)dbUpdated;

- (void)setAlbumContentArray:(NSArray *)aAlbumContentArray;
- (NSArray *)getAlbumContentArray;

- (void)setCurrentAlbumList:(NSArray *)aAlbumArray;
- (NSArray *)getCurrentAlbumList;

- (void)albumUpdateTrigger:(NSString *)msg;
- (void)streamUpdateTrigger;
- (void)radioLinkTrigger:(Radio *)radio;

- (MPDStatus *)getStatus;
- (int)getPosition;

- (void)setMainWindow:(NSWindow*)window;
- (NSWindow *)getMainWidow;

- (void)resetStatus;

@end
