/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "Preferences.h"
#import "Setting.h"


Setting *settingPref;

NSString *host;
NSString *port;
NSString *user;
NSString *pass;

NSString *hostHTTP;
NSString *argsPHP;

BOOL isValidPort = YES;
BOOL isValidIP = YES;


@implementation Preferences

- (instancetype)init {
    
    self = [super initWithWindowNibName:@"Preferences"];
    
    settingPref = [[Setting alloc] init];
    
    host = settingPref.host;

    NSString *prefport = [NSString stringWithFormat:@"%d", settingPref.port];
    port = prefport;
    
    user = settingPref.user;
    pass = settingPref.pass;
    
    hostHTTP = settingPref.hostHTTP;
    argsPHP = settingPref.argsPHP;
    
    return self;
}


- (void)awakeFromNib {
    [serverField setStringValue:host];
    [portField setStringValue:port];
    [usernameField setStringValue:user];
    [passwordField setStringValue:pass];
    
    if (hostHTTP != nil) {
        [checkButtonHTTP setState:NSControlStateValueOff];
        [serverHTTPField setEnabled:YES];
        [serverHTTPField setStringValue:hostHTTP];
    }
    
    if (argsPHP != nil) {
        [checkButtonPHP setState:NSControlStateValueOn];
        [argsPHPField setEnabled:YES];
        [argsPHPField setStringValue:argsPHP];
    }
    
    [self.prefwindow makeKeyAndOrderFront:nil];
    [self.prefwindow setLevel:NSStatusWindowLevel];
}


- (IBAction)save:(id)sender {
    
    host = [serverField stringValue];
    port = [portField stringValue];
    user = [usernameField stringValue];
    pass = [passwordField stringValue];
    
    hostHTTP = [serverHTTPField stringValue];
    argsPHP = [argsPHPField stringValue];
    
    if (host != nil && ![host isEqualToString:@""] && [settingPref isValidIP:host] == 1) {
        labelIP.textColor   = [NSColor labelColor];
        [settingPref setHost:host];
        isValidIP = YES;
    } else {
        labelIP.textColor = errorColor;
        isValidIP = NO;
    }
    
    // Vérifie qu'il n'y a que des chiffres
    NSCharacterSet *numberSet = [NSCharacterSet decimalDigitCharacterSet];
    NSString *trimmedString = [port stringByTrimmingCharactersInSet:numberSet];
    
    if (port != nil && ![port isEqualToString:@""] && ((trimmedString.length == 0) && (port.length > 0)) && [port integerValue] < 65536) {
        [settingPref setPort:[port floatValue]];
        labelPort.textColor = [NSColor labelColor];
        isValidPort = YES;
    } else {
        labelPort.textColor = errorColor;
        isValidPort = NO;
    }
    
    if (user != nil && ![user isEqualToString:@""])
        [settingPref setUser:user];
    else
        [settingPref unsetUser];

    
    if (pass != nil && ![pass isEqualToString:@""])
        [settingPref setPass:pass];
    else
        [settingPref unsetPass];
    

    if (checkButtonHTTP.state == NSControlStateValueOn) {
        
        [settingPref unsetHTTP];
        
    } else {
        
        if (hostHTTP != nil && ![hostHTTP isEqualToString:@""] && [settingPref isValidIP:hostHTTP] == 1) {
            [settingPref setHTTP:hostHTTP];
        } else {
            [settingPref unsetHTTP];
        }
    }
    
    if (checkButtonPHP.state == NSControlStateValueOff) {
        
        [settingPref unsetPHP];
        
    } else {
        
        if (argsPHP != nil && ![argsPHP isEqualToString:@""])
            [settingPref setPHP:argsPHP];
        else
            [settingPref unsetPHP];
    }
    
    if (isValidIP && isValidPort)
        [self.prefwindow close];
}


- (IBAction)cancel:(id)sender {
    [self.prefwindow close];
}


- (IBAction)buttonHTTP:(NSButton *)sender {
    [serverHTTPField setEnabled:checkButtonHTTP.state != NSControlStateValueOn];
}


- (IBAction)buttonPHP:(NSButton *)sender {
    [argsPHPField setEnabled:checkButtonPHP.state == NSControlStateValueOn];
}

@end
