/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#include <arpa/inet.h>
#import "Setting.h"


@implementation Setting

static Setting *sSettingManager = nil;

NSUserDefaults *userPref;

// Singleton
+ (Setting *)sSetting {
    if (sSettingManager == nil)
        sSettingManager = [[super allocWithZone:NULL] init];
    
    return sSettingManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sSetting];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}

- (id) init {
    
   self = [super init];
    
    userPref = [NSUserDefaults standardUserDefaults];

    NSString *prefhost = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSString class] fromData:[userPref objectForKey:@"host"] error:nil];
    if (prefhost != nil)
        host = prefhost;
    else
        host = @"127.0.0.1";
    
    NSString *prefport = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSString class] fromData:[userPref objectForKey:@"port"] error:nil];
    if (prefport != nil && ![prefport isEqualToString:@""]) {
        int i = [prefport intValue];
        port = i;
    } else {
        port = 6600;
    }
    
    NSString *prefuser = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSString class] fromData:[userPref objectForKey:@"user"] error:nil];
    if (prefuser != nil && ![prefuser isEqualToString:@""])
        user = prefuser;
    
    NSString *prefpass = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSString class] fromData:[userPref objectForKey:@"pass"] error:nil];
    if (prefpass != nil && ![prefpass isEqualToString:@""])
        pass = prefpass;
    
    NSString *prefHTTP = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSString class] fromData:[userPref objectForKey:@"HTTP"] error:nil];
    if (prefHTTP != nil && ![prefpass isEqualToString:@""])
        hostHTTP = prefHTTP;
    
    NSString *prefPHP = [NSKeyedUnarchiver unarchivedObjectOfClass:[NSString class] fromData:[userPref objectForKey:@"PHP"] error:nil];
    if (prefPHP != nil && ![prefpass isEqualToString:@""])
        argsPHP = prefPHP;
    
    return self;
}


- (NSString *)host {
    return host;
}

- (void)setHost:(NSString *)aHost {
        host = aHost;
    [userPref setObject:[NSKeyedArchiver archivedDataWithRootObject:host requiringSecureCoding:YES error:nil] forKey:@"host"];
    [userPref synchronize];
}


- (unsigned int)port {
    return port;
}

- (void)setPort:(unsigned int)aPort{
    port = aPort;
     [userPref setObject:[NSKeyedArchiver archivedDataWithRootObject:[NSString stringWithFormat:@"%d",port] requiringSecureCoding:YES  error:nil] forKey:@"port"];
    [userPref synchronize];
}


- (NSString *)user {
    if (user != nil) {
        return user;
    } else {
        NSString *vide = @"";
        return vide;
    }
}

- (void)setUser:(NSString *)aUser {
    user = aUser;
    [userPref setObject:[NSKeyedArchiver archivedDataWithRootObject:user requiringSecureCoding:YES error:nil] forKey:@"user"];
    [userPref synchronize];
}

- (void)unsetUser {
    user = nil;
    [userPref removeObjectForKey:@"user"];
    [userPref synchronize];
}


- (NSString *)pass {
    if (pass != nil) {
        return pass;
    } else {
        NSString *vide = @"";
        return vide;
    }
}

- (void)setPass:(NSString *)aPass {
    pass = aPass;
    [userPref setObject:[NSKeyedArchiver archivedDataWithRootObject:pass requiringSecureCoding:YES error:nil] forKey:@"pass"];
    [userPref synchronize];
}

- (void)unsetPass {
    pass = nil;
    [userPref removeObjectForKey:@"pass"];
    [userPref synchronize];
}


- (NSString *)hostHTTP {
    return hostHTTP;
}

- (void)setHTTP:(NSString *)aHTTP {
    hostHTTP = aHTTP;
    [userPref setObject:[NSKeyedArchiver archivedDataWithRootObject:hostHTTP requiringSecureCoding:YES error:nil] forKey:@"HTTP"];
    [userPref synchronize];
}

- (void)unsetHTTP {
    pass = nil;
    [userPref removeObjectForKey:@"HTTP"];
    [userPref synchronize];
}


- (NSString *)argsPHP {
    return argsPHP;
}

- (void)setPHP:(NSString *)aPHP {
    argsPHP = aPHP;
    [userPref setObject:[NSKeyedArchiver archivedDataWithRootObject:argsPHP requiringSecureCoding:YES error:nil] forKey:@"PHP"];
    [userPref synchronize];
}

- (void)unsetPHP {
    pass = nil;
    [userPref removeObjectForKey:@"PHP"];
    [userPref synchronize];
}


- (BOOL)isValidIP:(NSString *)ipAddress  {
    int success;
    
    const char *bar = [ipAddress UTF8String];
    
    struct in_addr dst;
    success = inet_pton(AF_INET, bar, &dst);
    if (success != 1) {
        struct in6_addr dst6;
        success = inet_pton(AF_INET6, bar, &dst6);
    }
    
    return success == 1;
}


@end
