/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "AlbumContentView.h"
#import "../../MPDService/MPDCommand.h"
#import "../../MPDService/Helper/MPDQueue.h"
#import "../../Model/Song.h"


@implementation AlbumContentView

MPDStatus *mpdStatusAC;

NSImage *logoPlayAC;
NSArray *albumContentArray;
NSString *currentAlbumName;

int currentStateAC;
int currentPositionAC;

long rowAlbumContentSelected;

- (void)viewDidLoad {
    [super viewDidLoad];
    
    albumContentArray = [[MPDCache sMPDCache] getAlbumContentArray];
    
    [_albumTableView setTarget:self];
    [_albumTableView setDoubleAction:@selector(doubleClick:)];
    
    [self setHead];
    [self updateStatus];
    [self statusListener];
}

- (void)awakeFromNib {
    [self performSelectorOnMainThread:@selector(updateTable:) withObject:_albumTableView waitUntilDone:NO];
}


- (void)doubleClick:(id)object {
    
    Song *song = albumContentArray[0];
    NSString * path = [song.fullPath stringByDeletingLastPathComponent];
    unsigned int rowNumber = (unsigned int) [_albumTableView clickedRow];

    [[MPDCommand sMPDCommand] playAlbum:song.album path:path position:rowNumber];
    currentPositionAC = -1;
}


- (void)updateTable:(NSTableView *)table {
    [table reloadData];
}


- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView {
    _albumTableView = tableView;
    return [albumContentArray count];
}


- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
    
    NSString *identifier = tableColumn.identifier;
    NSTableCellView *cell = [tableView makeViewWithIdentifier:identifier owner:self];
    
    int state = mpdStatusAC.state;
    
    NSArray *songs = [self getSongs:row];
    
    Song *selectedSong = songs[0];
    
    if ([identifier isEqualToString:@"trackNumber"])
        return [self setTrack:cell row:row songs:songs state:state];
    else if ([identifier isEqualToString:@"title"])
        return cell.textField.stringValue = [NSString stringWithFormat:@"%@",selectedSong.title];
    else if ([identifier isEqualToString:@"time"])
        return [self setTime:cell songs:songs state:state];
    else
        return nil;
}


- (NSArray *)getSongs:(NSInteger)row {
    
    Song *selectedSong = albumContentArray[row];
    
    Song *song = [[Song alloc]init];
    [song setTrack:selectedSong.track];
    [song setTitle:selectedSong.title];
    [song setArtist:selectedSong.albumArtist];
    [song setAlbum:selectedSong.album];
    [song setDiskNb:selectedSong.diskNb];
    [song setDuration:selectedSong.duration];
    
    Song *statusSong = [[Song alloc]init];
    [statusSong setTitle:mpdStatusAC.title];
    [statusSong setArtist:mpdStatusAC.albumArtist];
    [statusSong setAlbum:mpdStatusAC.album];
    
    NSString *artist = selectedSong.albumArtist;
    
    if ([artist isEqualToString:@""]) {
        [statusSong setArtist:mpdStatusAC.artist];
        [song setArtist:selectedSong.artist];
    }
    
    NSMutableArray *songs = [NSMutableArray array];
    [songs addObject:song];
    [songs addObject:statusSong];
    
    return songs;
}


- (NSString *)setTrack:(NSTableCellView *)cell row:(NSInteger)row songs:(NSArray *)songs state:(int)state {
    
    Song *selectedSong = songs[0];
    Song *statusSong = songs[1];
    
    if ([statusSong.title isEqualToString:selectedSong.title] &&
        [statusSong.artist isEqualToString:selectedSong.artist] &&
        [statusSong.album isEqualToString:selectedSong.album] &&
        (state != MPD_STATE_STOP)) {
        
        currentPositionAC = (int)row;
        
        cell.imageView.image = logoPlayAC;
        cell.textField.stringValue =@"";
        
    } else {
        
        cell.imageView.image = nil;;
        
        if (selectedSong.diskNb != 0)
            cell.textField.stringValue = [NSString stringWithFormat:@"%d ∙ %d",selectedSong.diskNb, selectedSong.track];
        else
            cell.textField.stringValue = [NSString stringWithFormat:@"%d", selectedSong.track];
    }
    
    return cell.textField.stringValue;
    
}


- (NSString *)setTime:(NSTableCellView *)cell songs:(NSArray *)songs state:(int)state {
    
    Song *selectedSong = songs[0];
    Song *statusSong = songs[1];
    
    int minutes = selectedSong.duration / 60;
    int seconds = (int)selectedSong.duration % 60;
    NSString *time = [NSString stringWithFormat:@"%2d:%02d",minutes, seconds];
    
    if ([statusSong.title isEqualToString:selectedSong.title] && [statusSong.artist isEqualToString:selectedSong.artist] && (state != MPD_STATE_STOP))
        cell.textField.textColor = [NSColor colorWithRed:255.0/255.0 green:171.0/255.0 blue:1.0/255.0 alpha:1.0];
    else
        cell.textField.textColor = [NSColor secondaryLabelColor];
    
    return cell.textField.stringValue = [NSString stringWithFormat:@"%@",time];
}


- (void)setHead {
    
    Song *song = albumContentArray[0];
    albumName.stringValue = song.album;
    
    NSString *albumArtist;
    
    if (song.albumArtist.length != 0)
        albumArtist = song.albumArtist;
    else
        albumArtist = song.artist;
    
    artistName.stringValue = [NSString stringWithFormat:@"%@", albumArtist];
    genreAndYear.stringValue = [NSString stringWithFormat:@"%@ ∙ %d", song.genre, song.date];
    
    [self setHeadTime];
    [self setHeadLogo];
}


- (void)setHeadTime {
    
    float duration = 0;
    
    for (Song *song in albumContentArray)
        duration = song.duration + duration;
    
    int hour = duration / (60*60);
    int minute = (int)(duration/60) % 60;
    
    NSString *temps;
    
    if (hour > 0 && minute > 0)
        temps = [NSString stringWithFormat:@"%dh %dmn", hour, minute];
    else if (hour > 0 && minute == 0)
        temps = [NSString stringWithFormat:@"%dh", hour];
    else
        temps = [NSString stringWithFormat:@"%dmn", minute];
    
    durationText.stringValue = [NSString stringWithFormat:@"%@", temps];
}


- (void)setHeadLogo {
    
    Song *song = albumContentArray[0];
    
    currentAlbumName = song.album;
    
    if (song.bits > 16)
        [logoAudio setImage:[NSImage imageNamed:@"hires"]];
    
    NSString *extension = song.extension;
    
    if ([extension isEqualToString:@"dsf"] || [extension isEqualToString:@"dff"])
        [logoAudio setImage:[NSImage imageNamed:@"dsd"]];
    
    
    NSShadow *shadow = [[NSShadow alloc] init];
    [shadow setShadowBlurRadius:1.5f];
    [shadow setShadowOffset:CGSizeMake(0.5f, -1.5f)];
    
    NSColor *myColor = [NSColor colorWithRed:0 green:0 blue:0 alpha:0.6f];
    [shadow setShadowColor:myColor];
    [logoAudio setShadow:shadow];
}


- (void)statusListener {
    
    [[NSNotificationCenter defaultCenter] addObserverForName:@"statusChanged" object:nil queue:nil usingBlock:^(NSNotification *note) {
        
        [self updateStatus];
        
    }];
}


-(void)updateStatus {
    mpdStatusAC = [[MPDCache sMPDCache] getStatus];
    
    if (currentStateAC != mpdStatusAC.state) {
        
        currentStateAC = mpdStatusAC.state;
        
        if (mpdStatusAC.state == MPD_STATE_PLAY)
            logoPlayAC = [NSImage imageNamed:@"state_play"];
        else if (mpdStatusAC.state == MPD_STATE_PAUSE)
            logoPlayAC = [NSImage imageNamed:@"state_pause"];
        else
            logoPlayAC = nil;
        
        [_albumTableView reloadDataForRowIndexes:[NSIndexSet indexSetWithIndex:currentPositionAC] columnIndexes:[NSIndexSet indexSetWithIndex:0]];
    }
    
    if ([mpdStatusAC.album isEqual:currentAlbumName] && currentPositionAC != mpdStatusAC.titlePosition)
        [self updateTable:_albumTableView];
    
}

- (void)menuWillOpen:(NSMenu *)menu {
    rowAlbumContentSelected = (long)_albumTableView.clickedRow;
}


- (IBAction)addToQueue:(id)sender {
    NSIndexSet *row = [_albumTableView selectedRowIndexes];
        
    NSMutableArray *selectedItems = [NSMutableArray array];
    [row enumerateIndexesUsingBlock:^(NSUInteger idx, BOOL *stop) {
        [selectedItems addObject:[NSNumber numberWithInteger:idx]];
    }];
    
    NSMutableArray *listSong = [NSMutableArray array];
    
    if ([selectedItems count] > 1) {
        
        for (NSString *string in selectedItems) {
            int position = [string intValue];
            [listSong addObject:albumContentArray[position]];
        }
        
    } else {
        
        [listSong addObject:albumContentArray[rowAlbumContentSelected]];
    }
    
    [[MPDQueue sMPDQueue] addToQueue:[listSong copy]];
}

@end
