//
//  RadioItem.m
//  MPD Client
//
//  Created by icare on 27/02/2019.
//  Copyright © 2019 Old-Geek Inc. All rights reserved.
//

#include <UIImageView+WebCache.h>
#import "RadioItem.h"
#import "RadioOverCover.h"
#import "../../Model/Radio.h"
#import "../../MPDService/MPDCommand.h"
#import "../../MPDService/MPDCache.h"
#import "../../MPDService/Helper/MPDStream.h"


@implementation RadioItem

NSTimer *doubleClickTimer;

BOOL isDoubleClick;


- (void)setRepresentedObject:(id)representedObject{
    [super setRepresentedObject:representedObject];
    
    if (representedObject !=nil) {
        NSString *imageValue = [representedObject objectForKey:@"logo"];
        NSString *label = [representedObject objectForKey:@"name"];
        NSString *subtext = [representedObject objectForKey:@"subtext"];
        NSString *link = [representedObject objectForKey:@"isLink"];

        [playMask setHidden:[link isEqualToString:@"yes"]];
        
        [labelRadio setStringValue:label];
        [subTextRadio setStringValue:subtext];
        
        NSURL* url = [NSURL URLWithString:imageValue];
        
        if ([link isEqualToString:@"no"]) {
            
            if ([imageValue isEqualToString:@"null"])
                logoRadio.image = [NSImage imageNamed:@"no_cover"];
            else {
                [logoRadio sd_setImageWithURL:url placeholderImage:[UIImage imageNamed:@"no_cover"]];
            }
            
            NSShadow *shadow = [[NSShadow alloc] init];
            [shadow setShadowBlurRadius:1.5f];
            [shadow setShadowOffset:CGSizeMake(0.5f, -1.5f)];
            
            NSColor *myColor = [NSColor colorWithRed:0 green:0 blue:0 alpha:0.6f];
            [shadow setShadowColor:myColor];
            [logoRadio setShadow:shadow];
            
        } else {
            logoRadio.image = [NSImage imageNamed:@"NSFolder"];
        }
    }
}


- (IBAction)clickPlay:(NSButton *)sender {
    
    NSString *url = [[sender.target representedObject] objectForKey:@"url"];
    NSString *stream = [self getStreamURL:url];
        
    if (stream.length > 7)
        [[MPDCommand sMPDCommand] playStream:stream];
}


- (IBAction)clickFolder:(NSButton *)sender {

    NSString *url = [[sender.target representedObject] objectForKey:@"url"];
    NSString *name = [[sender.target representedObject] objectForKey:@"name"];
    if (isDoubleClick) {
        
        isDoubleClick = NO;
        
        Radio *radio = [[Radio alloc] init];
        [radio setName:name];
        [radio setUrl:url];
        
        [[MPDCache sMPDCache] radioLinkTrigger:radio];
        
    } else if (!isDoubleClick) {
        
        isDoubleClick = YES;
        doubleClickTimer = [NSTimer scheduledTimerWithTimeInterval:0.3 target:self selector:@selector(TimerReload:) userInfo:nil repeats:NO];
    }
}

-(void)TimerReload:(NSTimer *)timer {
    doubleClickTimer = nil;
    isDoubleClick = NO;
}


- (IBAction)addToStream:(NSButton *)sender {
    NSString *url = [[sender.target representedObject] objectForKey:@"url"];
    NSString *name = [[sender.target representedObject] objectForKey:@"name"];
    NSString *subtext = [[sender.target representedObject] objectForKey:@"subtext"];
    NSString *stream = [self getStreamURL:url];
    
    if (stream.length > 7) {
        [[MPDStream sMPDStream] addStream:[NSString stringWithFormat:@"%@#%@", stream, name]];
        
        NSString *message = [NSString stringWithFormat:@"%@\n\n%@", name, subtext];
        [self toast:@"Stream ajouté" message:message];
    }
}

- (void)toast:(NSString *)msg message:(NSString *)message {
    
    NSAlert *alert = [[NSAlert alloc] init];
    [alert addButtonWithTitle:@"Ok"];
    NSButton *button =  [[alert buttons] objectAtIndex:0];
    [button setHidden:YES];
    [alert setMessageText:msg];
    [alert setInformativeText:message];
    [alert setAlertStyle:NSAlertStyleInformational];
    [alert beginSheetModalForWindow:[[MPDCache sMPDCache] getMainWidow] completionHandler:nil];
    
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        sleep(2);
        dispatch_sync(dispatch_get_main_queue(), ^{
            [alert.window close];
        });
    });
}


-(NSString *)getStreamURL:(NSString *)url {
    
    NSURL *nsurl = [NSURL URLWithString:url];
    NSData *data = [NSData dataWithContentsOfURL:nsurl];
    NSString *dataString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    // rustine, ligne double
    NSRange r2 = [dataString rangeOfString:@"\n"];
    NSString *final = [dataString substringToIndex:((r2.location - 1) + r2.length)];
    
    NSString *extension = [final pathExtension].lowercaseString;
    
    NSString *urlToPlay = final;
    
    if ([extension isEqualToString:@"pls"]) {
        urlToPlay = [self openFilePLS:final];
    } else if ([extension isEqualToString:@"m3u"] || [extension isEqualToString:@"m3u8"]) {
        urlToPlay = [self openFileM3U:final];
    } else if ([extension isEqualToString:@"asx"]) {
        urlToPlay = [self openFileASX:final];
    }
    
    return urlToPlay;
}


- (NSString *)openFilePLS:(NSString *)playlist {
    
    NSURL *url = [NSURL URLWithString:playlist];
    NSData *data = [NSData dataWithContentsOfURL:url];
    NSString *dataString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    NSArray *list = [dataString componentsSeparatedByString:@"\n"];
    
    for (NSString *str in list) {
        
        if ([str containsString:@"File"] && [str containsString:@"http://"]) {
            
            NSRange r1 = [str rangeOfString:@"http://"];
            NSString *startURL = [str substringFromIndex:(r1.location)];
            
            return startURL;
        }
    }
    
    return nil;
}

- (NSString *)openFileM3U:(NSString *)playlist {
    
    NSURL *url = [NSURL URLWithString:playlist];
    NSData *data = [NSData dataWithContentsOfURL:url];
    NSString *dataString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    NSArray *list = [dataString componentsSeparatedByString:@"\n"];
    
    for (NSString *str in list) {
        
        if ([str containsString:@"http://"])
            return str;
    }
    
    return nil;
}

- (NSString *)openFileASX:(NSString *)playlist {
    
    NSURL *url = [NSURL URLWithString:playlist];
    NSData *data = [NSData dataWithContentsOfURL:url];
    NSString *dataString = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    NSArray *list = [dataString componentsSeparatedByString:@"\n"];
    
    for (NSString *str in list) {
        
        if ([str containsString:@"<Ref href = \""] && [str containsString:@"http://"]) {
            
            NSRange r1 = [str rangeOfString:@"http://"];
            NSString *startURL = [str substringFromIndex:(r1.location)];
            
            NSRange r2 = [startURL rangeOfString:@"\""];
            NSString *final = [startURL substringToIndex:((r2.location-1) + r2.length)];
                        
            return final;
            
        }
    }
    
    return nil;
}


@end
