/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

/* *******************************************************************
 200 : succès de la requête ;
 301 et 302 : redirection, respectivement permanente et temporaire ;
 401 : utilisateur non authentifié ;
 403 : accès refusé ;
 404 : page non trouvée ;
 500 et 503 : erreur serveur ;
 504 : le serveur n'a pas répondu.
 ******************************************************************* */

#import "RadioLoader.h"
#import "../../Model/Radio.h"
#import "../../Tools/NSString+HTML.h"


@implementation RadioLoader

static RadioLoader *sRadioLoaderManager = nil;


// Singleton
+ (RadioLoader *)sRadioLoader {
    
    if (sRadioLoaderManager == nil)
        sRadioLoaderManager = [[super allocWithZone:NULL] init];
    
    return sRadioLoaderManager;
}

+ (id)allocWithZone:(NSZone *)zone {
    return [self sRadioLoader];
}

- (id)copyWithZone:(NSZone *)zone {
    return self;
}


- (id) init {
    self = [super init];
    return self;
}

- (NSArray *)getIndex {
    
    NSURL *url = [NSURL URLWithString:@"http://opml.radiotime.com/"];
    NSData *data = [NSData dataWithContentsOfURL:url];
    NSString *resultat = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    return [self radioList:resultat];
}


- (NSArray *)getListOf:(NSString *)aURL {
    
    NSURL *url = [NSURL URLWithString:aURL];
    NSData *data = [NSData dataWithContentsOfURL:url];    
    NSString *resultat = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    if (resultat.length < 1 || [resultat containsString:@"403 - Forbidden: Access is denied"]) {
        
        Radio *radio = [[Radio alloc]init];
        [radio isLink:@"yes"];
        [radio setUrl:@"http://127.0.0.1"];
        [radio setName:@"403 - Forbidden: Access is denied"];
        
        return [NSArray arrayWithObject:radio];
    }
    
    return [self tri:resultat];
}


- (NSArray *)search:(NSString *)text {
    
    NSString *query = [text stringByReplacingOccurrencesOfString:@" " withString:@"%20"];
    query = [query stringByReplacingOccurrencesOfString: @"&" withString: @"&amp;"];
    query = [query stringByReplacingOccurrencesOfString: @"'" withString: @"%27"];
    
    NSString *recherche = [NSString stringWithFormat:@"http://opml.radiotime.com/Search.ashx?query=%@", query];
    NSURL *url = [NSURL URLWithString:recherche];
    NSData *data = [NSData dataWithContentsOfURL:url];
    NSString *resultat = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
    
    return [self radioList:resultat];
}

- (NSArray *)radioList:(NSString *)string {
    
    if (string.length < 1 || [string containsString:@"403 - Forbidden: Access is denied"])
        return [NSArray arrayWithObject:[self error:string]];
        
    return [self tri:string];
}


- (NSArray *)tri:(NSString *)string {
    
    NSArray *array = [self regimeSec:string];
    
    NSMutableArray *radioList = [NSMutableArray array];
    
    for (NSString *string in array) {
        
        if ([string containsString:@"type=\"link\""])
            [radioList addObject:[self moulinetteLien:string]];
        else if ([string containsString:@"type=\"audio\""])
            [radioList addObject:[self moulinetteNode:string]];
    }
    
    return radioList;
}


- (Radio *)moulinetteLien:(NSString *)string {
    
    NSArray *array = [self decoupe:string];
    
    Radio *radio = [[Radio alloc]init];
    
    [radio isLink:@"yes"];
    
    for (NSString *str in array) {
        
        [radio setImage:@"null"];
        
        if ([str containsString:@"URL="])
            [radio setUrl:[self getUrl:str]];
        
        if ([str containsString:@"radioname="])
            [radio setName:[self getText:str]];
    }
    
    return radio;
}

- (Radio *)moulinetteNode:(NSString *)string {
    
    NSArray *array = [self decoupe:string];
    
    Radio *radio = [[Radio alloc]init];
    
    [radio isLink:@"no"];
    
    for (NSString *str in array) {
        if ([str containsString:@"URL="])
            [radio setUrl:[self getUrl:str]];
        
        if ([str hasPrefix:@"radioname="])
            [radio setName:[self getText:str]];
        
        if ([str containsString:@"subtext="])
            [radio setDescription:[self getSubText:str]];
        
        if ([str containsString:@"image="])
            [radio setImage:[self getImage:str]];

    }
    
    return radio;
}

- (NSArray *)decoupe:(NSString *)string {
    
    NSString *str = [string stringByReplacingOccurrencesOfString:@"<outline " withString:@""];
    str = [str stringByReplacingOccurrencesOfString:@"/>" withString:@""];
    str = [str stringByReplacingOccurrencesOfString:@"\" " withString:@"\"\n"];
    
    return [str componentsSeparatedByString:@"\n"];
}

- (NSArray *)regimeSec:(NSString *)data {
    
    NSString *slim = data;
    
    slim = [slim stringByReplacingOccurrencesOfString:@"&amp;" withString:@"&"];
    
    // evite les bug avec subtext
    slim = [slim stringByReplacingOccurrencesOfString:@"\" text=\"" withString:@"\" radioname=\""];
    
    if ([slim containsString:@"<status>200</status>"]) {
        
        NSRange range = [slim rangeOfString:@"<outline"];
        slim = [slim substringFromIndex:range.location];
        NSRange range2 = [slim rangeOfString:@"</body>"];
        slim = [slim substringToIndex:range2.location];
        
        return [slim componentsSeparatedByString:@"\n"];
    }
    
    return nil;
}


- (NSString *)getUrl:(NSString *)aLine {
    
    NSRange r1 = [aLine rangeOfString:@"URL=\""];
    NSString *startURL = [aLine substringFromIndex:(r1.location + r1.length)];
    NSRange r2 = [startURL rangeOfString:@"\""];
    NSString *final = [startURL substringToIndex:((r2.location-1) + r2.length)];
    
    if (final == (id)[NSNull null] || final.length == 0 )
        final = @"null0";
    
    return final;
}

- (NSString *)getText:(NSString *)aLine {
    
    aLine = aLine.htmlDecode;
    
    NSRange r1 = [aLine rangeOfString:@"radioname=\""];
    NSString *name = [aLine substringFromIndex:(r1.location + r1.length)];
    NSRange r2 = [name rangeOfString:@"\""];
    NSString *final = [name substringToIndex:((r2.location-1) + r2.length)];
    
    if (final == (id)[NSNull null] || final.length == 0 )
        final = @"null1";
    
    return final;
}

- (NSString *)getSubText:(NSString *)aLine {
    
    aLine = aLine.htmlDecode;
    
    NSRange r1 = [aLine rangeOfString:@"subtext=\""];
    NSString *sub = [aLine substringFromIndex:(r1.location + r1.length)];
    NSRange r2 = [sub rangeOfString:@"\""];
    NSString *final = [sub substringToIndex:((r2.location-1) + r2.length)];
    
    return final;
}

- (NSString *)getImage:(NSString *)aLine {
    
    NSRange r1 = [aLine rangeOfString:@"image=\""];
    NSString *img = [aLine substringFromIndex:(r1.location + r1.length)];
    NSRange r2 = [img rangeOfString:@"\""];
    NSString *final = [img substringToIndex:((r2.location-1) + r2.length)];
    
    return final;
}


- (Radio *)error:(NSString *)string {
    Radio *radio = [[Radio alloc]init];
    
    [radio isLink:@"yes"];
    [radio setUrl:@"0"];
    
    if (string.length > 1 || [string containsString:@"403 - Forbidden: Access is denied"])
        [radio setName:@"403 - Forbidden: Access is denied"];
    else
        [radio setName:@"Erreur de connexion"];
    
    return radio;
    
}

@end
