/*
 * MPDclient - Music Player Daemon (MPD) remote for Mac OS
 * Copyright (C) 2019  Old-Geek
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#import "RadioView.h"
#import "RadioItem.h"
#import "RadioLoader.h"
#import "RadioState.h"
#import "../../Model/Radio.h"


@implementation RadioView

NSNib *itemRadio = nil;

BOOL addHistoric = YES;
BOOL clickLocked = NO;


- (id)init {
    self = [super init];
    
    [self startProgress];
    
    [MPDCache sMPDCache].delegateRadio = self;
    
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        
        if (RadioState.radioArrayTableIndex == nil)
            RadioState.radioArrayTableIndex = [[RadioLoader sRadioLoader] getIndex];
        
        if (RadioState.radioNavigationCache == nil)
            RadioState.radioNavigationCache = [NSMutableDictionary dictionary];
        
        dispatch_sync(dispatch_get_main_queue(), ^{
         
            if (RadioState.radioNavigationHistorique == nil)
                RadioState.radioNavigationHistorique = [NSMutableArray array];
            
            [self awakeFromNib];
            
            Radio *radio = RadioState.radioArrayTableIndex[0];
            [self getURLContent:radio.radioUrl name:radio.radioName];
            [self stopProgress];
        });
    });
    
    return self;
}


- (void)awakeFromNib {
    [radioSearchField setSendsWholeSearchString:YES];
    
    [_radioTableView setTarget:self];
    [_radioTableView setAction:@selector(onIndexClick:)];
    [self performSelectorOnMainThread:@selector(updateTable:) withObject:_radioTableView waitUntilDone:NO];
    [_radioTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
}


- (void)updateTable:(NSTableView *)table {
    [table reloadData];
    [_radioTableView selectRowIndexes:[NSIndexSet indexSetWithIndex:0] byExtendingSelection:NO];
}


- (NSInteger)numberOfRowsInTableView:(NSTableView *)tableView {
    _radioTableView = tableView;
    return [RadioState.radioArrayTableIndex count];
}


- (id)tableView:(NSTableView *)tableView objectValueForTableColumn:(NSTableColumn *)tableColumn row:(NSInteger)row {
    
    Radio *string = RadioState.radioArrayTableIndex[row];
    if ([tableColumn.identifier isEqualToString:@"name"]) {
        
        NSTableCellView *cell = [tableView makeViewWithIdentifier:@"name" owner:self];
        
        NSString *str = [NSString stringWithFormat:@"  %@", string.radioName];
        
        if (string.radioName != NULL)
            cell.textField.stringValue = str;
        else
            cell.textField.stringValue = @"";
        
        return cell;
    }
    
    return nil;
}


// Sidebar clicked
- (void)onIndexClick:(id)object {
    
    int rowNumber = (int) [_radioTableView clickedRow];
    
    if (rowNumber < [RadioState.radioArrayTableIndex count]) {
        Radio *radio = RadioState.radioArrayTableIndex[rowNumber];
        [self getURLContent:radio.radioUrl name:radio.radioName];
    }
}


// RadioItem Click
- (void)onRadioLinkClick:(Radio *)radio {
    if (!clickLocked) {
        NSString *name = radio.radioName;
        NSString *url = radio.radioUrl;
        [self getURLContent:url name:name];
    }
}


- (void)onSearch:(NSString *)query {
    NSString *newQuery = query;
    
    unsigned long count = newQuery.length;
    
    if (count > 1) {
        
        [self startProgress];
        
        NSString *titre = [NSString stringWithFormat:@"Recherche: %@", newQuery];
        titreCollectionView.stringValue = titre;
        itemRadio = [[NSNib alloc] initWithNibNamed:@"RadioItem" bundle:nil];
        
        dispatch_async(dispatch_get_global_queue(0, 0), ^{
            
            NSArray *getAll = [[RadioLoader sRadioLoader] search:newQuery];
            RadioState.contentRadio = getAll;
            
            dispatch_sync(dispatch_get_main_queue(), ^{
                [self showList];
            });
        });
        
    }
}


- (void)getURLContent:(NSString *)url name:(NSString *)name {
    
    [self startProgress];
    
    titreCollectionView.stringValue = name;
    
    itemRadio = [[NSNib alloc] initWithNibNamed:@"RadioItem" bundle:nil];
    
    dispatch_async(dispatch_get_global_queue(0, 0), ^{
        
        NSArray *getCache = RadioState.radioNavigationCache[url];
        
        if (getCache != NULL) {
            
            RadioState.contentRadio = getCache;
            
        } else {

            RadioState.contentRadio = [[RadioLoader sRadioLoader] getListOf:url];
            RadioState.radioNavigationCache[url] = RadioState.contentRadio;
        }
        
        dispatch_sync(dispatch_get_main_queue(), ^{
            
            [self showList];
            
            if (addHistoric)
                [RadioState.radioNavigationHistorique addObject:@{@"url":url, @"name":name}];
            
            addHistoric = YES;
        });
    });
    
}

- (void)showList {
    
    NSMutableArray *firstlist = [NSMutableArray array];
    
    for (Radio *obj in RadioState.contentRadio) {
        
        NSString *rname = obj.radioName;
        if (rname.length == 0 )
            rname = @"";
        NSString *rsubtext = obj.radioDescription;
        if (rsubtext.length == 0 )
            rsubtext = @"";
        NSString *rurl = obj.radioUrl;
        if (rurl.length == 0 )
            rurl = @"";
        NSString *rlogo = obj.radioLogoUrl;
        if (rlogo.length == 0 )
            rlogo = @"null";
        
        [firstlist addObject:@{@"name":rname, @"subtext":rsubtext, @"url":rurl, @"logo":rlogo, @"isLink":obj.link}];
    }
    
    [self->_radioCollectionView registerNib:itemRadio forItemWithIdentifier:@"RadioItem"];
    [self->_radioCollectionView setContent:firstlist];
    
    [self->_radioCollectionView reloadData];
    
    [self->_radioCollectionView setNeedsDisplay:YES];
    [self stopProgress];
    
}


- (void)startProgress {
    clickLocked = YES;
    [progressCircle setHidden:NO];
    [progressCircle setIndeterminate:YES];
    [progressCircle setUsesThreadedAnimation:YES];
    [progressCircle startAnimation:nil];
}


- (void)stopProgress {
    clickLocked = NO;
    [self->progressCircle stopAnimation:nil];
    [self->progressCircle setHidden:YES];
}


- (IBAction)search:(id)sender {
    NSString *query = [sender stringValue];
    [self onSearch:query];
}


- (IBAction)previousButton:(id)sender {
    
    int count = (int)[RadioState.radioNavigationHistorique count];
        
    if (count > 1) {
        
        NSString *name = [RadioState.radioNavigationHistorique[(count-2)] objectForKey:@"name"];
        NSString *url = [RadioState.radioNavigationHistorique[(count-2)] objectForKey:@"url"];
        [RadioState.radioNavigationHistorique removeObjectAtIndex:(count-1)];
        
        addHistoric = NO;
        
        [self getURLContent:url name:name];
    }
}


@end
